/* Leif Svalgaard, 2006 */
#include <stdio.h>
#include <string.h>
#include <math.h>

#define KSIZE 16
void AES (const char *Operation, /* EK, E, DK, D  */
          const char *Key,       /* 16-char block */
          char *Key_Schedule,    /* 176-char area */
          char *Plain_Text,      /* 16-char block */
          char *Cipher_Text);    /* 16-char block */

char Schedule[21*KSIZE];
char Key   [KSIZE];
char Plain [KSIZE];
char Chain [KSIZE];
char Cipher[KSIZE];

void Get_Binary (char Binary[], const char Hex[])
{   /* Convert the Hex to Binary format */
    int i, j;
    int b, bin;

    j = 0;
    for (i=0; i<2*KSIZE; i++) {
        switch (Hex[i]) {
          case '0':           b =  0; break;
          case '1':           b =  1; break;
          case '2':           b =  2; break;
          case '3':           b =  3; break;
          case '4':           b =  4; break;
          case '5':           b =  5; break;
          case '6':           b =  6; break;
          case '7':           b =  7; break;
          case '8':           b =  8; break;
          case '9':           b =  9; break;
          case 'a': case 'A': b = 10; break;
          case 'b': case 'B': b = 11; break;
          case 'c': case 'C': b = 12; break;
          case 'd': case 'D': b = 13; break;
          case 'e': case 'E': b = 14; break;
          case 'f': case 'F': b = 15; break;
          default: /* bad */  b = -1; break;
        }

        if ((i & 1) == 1)
            Binary[j++] = bin*16 + b;
        else
            bin = b;

    } /* end for */
}

char Hidden_Key[KSIZE] = {0x3f,0xed,0x18,0xf6,0xea,0xd1,0xb4,0x46,
                          0xbf,0xda,0xa2,0x26,0x57,0x53,0x72,0x05};

char Recovered_Key[KSIZE];
void Recover (char *Out, const char *In, const char *Hidden)
{ /* Decrypt with Private Key. For now, just XOR */
    int i;
    for (i=0; i<KSIZE; i++)
        Out[i] = (In[i]) ^ (Hidden[i]);
}

void XOR_Block (char *Out, const char *In)
{ /* XOR with a given block of bits */
    int i;
    for (i=0; i<KSIZE; i++)
        Out[i] = (Out[i]) ^ (In[i]);
}

#define FILL ' '
#define NUL '\0'
#define NL '\n'
#define LBUF 80
char Buf[LBUF];
FILE *File_In;
FILE *File_Out;

union {
  char Input[256];
  struct {
    char Mark;
    char Blank;
    char Cipher[2*KSIZE];
  } Text;
} Record;

void main(int argc, char *argv[])
{   /* Decrypt Exit Program Info */
    char *File_Name;
    int Chunk;
    char *Buf_Chunk;

    /* printf ("%s%s%s%s%s%s%s",
        "program name = [", argv[0],
      	 "]\nfile name1= [", argv[1],
      	 "]\nfile name2= [", argv[2],
        "]\n"); getchar ();
    */

    if ((File_In = fopen (argv[1], "r")) == NULL) if
       ((File_In = fopen ("LSVALGAARD/QCSRC(EUSRPWD)", "r")) == NULL)
        return;

    if ((File_Out = fopen (argv[2], "w")) == NULL) if
       ((File_Out = fopen ("LSVALGAARD/QCSRC(DUSRPWD)","w")) == NULL)
       {fclose (File_In); return;}

    /* Hidden_Key.Upper = sin (2); */
    /* Hidden.Key.Lower = cos (2); */

    while ((fgets (Record.Input, 256, File_In)) != NULL) {
        if (Record.Text.Mark == 'K') { /* Get Encrypted Key */
            Get_Binary (Cipher, Record.Text.Cipher);
            memcpy  (Chain, Cipher, KSIZE);
            Recover (Key, Cipher, Hidden_Key);
            AES ("DK", Key, Schedule, NULL, NULL);
            Chunk = 0;
        }
        else
        if (Record.Text.Mark == 'A') { /* AES Decrypt */
            Get_Binary (Cipher, Record.Text.Cipher);
            AES ("D", NULL, Schedule, Plain, Cipher);
            XOR_Block (Plain, Chain);
            memcpy (Chain, Cipher, KSIZE);

            Buf_Chunk = &Buf[Chunk];
            memcpy (Buf_Chunk, Plain, KSIZE);
            Chunk = Chunk + KSIZE;
            if (Chunk >= LBUF) {
                Buf[LBUF-2] = NL; Buf[LBUF-1] = NUL;
                fputs (Buf, File_Out);
                Chunk = 0;
            }
        }
    } /* end While: reading */

    fclose (File_In);
    fclose (File_Out);
    return;

}
